"""
    Module for doing file searching and replacing stuff.
"""

import os

from . import fileutils


def main():
    """Main function for search and replace."""
    args = fileutils.get_search_args(require_replace_args=True)
    files = []
    for file in fileutils.iter_matching_files(
        cur_dir=args.cur_dir,
        file_pattern=args.file_pattern,
        text_search_string=args.search_string,
    ):
        files.append(file)
        with open(
            file, encoding="utf-8"
        ) as fd:  # pylint: disable=invalid-name,duplicate-code
            file_data = fd.read()
        matches = []
        for i, line in enumerate(file_data.splitlines()):
            if args.search_string in line:
                matches.append((i, line))

        print(f"Found {len(matches)} matches in {os.path.abspath(file)}:")

        for match in matches:
            haystack = match[1].strip()
            # pos = haystack.find(match)
            # if len(haystack) > 60:
            #  haystack = haystack[0:60] + '...'
            print(f"  {match[0]}: {haystack}")

    if "y" == input("Apply replace? (y/n): ").lower():
        print(f"Replacing now... {len(files)}")
        for file in files:
            print(f"Replace in file {file} ")
            fileutils.replace_in_file(
                file_path=file,
                search_text=args.search_string,
                replace_text=args.replace_string,
            )


if __name__ == "__main__":
    main()



"""
    Helper function to index a file system
"""

import os
import sys


def file_size(file: str) -> str:
    """Return file size as a string"""
    try:
        val = os.stat(file).st_size
        return f"{val}"
    except PermissionError:
        return "-1"
    except FileNotFoundError:
        return "-1"


def _main():
    try:
        for root, _, files in os.walk("/", topdown=False):
            for name in files:
                file = os.path.abspath(os.path.join(root, name))
                print(f'"{file}", {file_size(file)}')
    except KeyboardInterrupt:
        sys.exit(1)


if __name__ == "__main__":
    _main()



# pylint: skip-file
"""
    Module for doing file searching stuff.
"""
import os
from . import fileutils


def main():
    """Main program"""
    args = fileutils.get_search_args()
    for file in fileutils.iter_matching_files(
        cur_dir=args.cur_dir,
        file_pattern=args.file_pattern,
        text_search_string=args.search_string,
    ):
        with open(file, encoding="utf-8") as fd:
            file_data = fd.read()
        matches = []
        for i, line in enumerate(file_data.splitlines()):
            if args.search_string in line:
                matches.append((i, line))
        print(f"Found {len(matches)} matches in {os.path.abspath(file)}:\n")
        for match in matches:
            haystack = match[1].strip()
            print(f"  {match[0]}: {haystack}" % (match[0], haystack))


if __name__ == "__main__":
    main()



"""
Generates a list of the top directories and files that are taking up space.
The search is started from the cwd.
"""
# pylint: skip-file


import os
import sys
from typing import Any, Dict, List


def add_path(tree: dict, path_list: List[str], size: int) -> None:
    curr_tree: dict = tree
    for p in path_list:
        curr_tree["size"] += size
        subtree = curr_tree["children"].get(p, None)
        if subtree is None:
            subtree = dict(name=p, size=size, children={})
            curr_tree["children"][p] = subtree
        curr_tree = subtree
    curr_tree["size"] += size


def split_paths(path: str) -> List[str]:
    out: List = []
    curr_path = path
    while True:
        head, leaf = os.path.split(curr_path)
        if leaf == "":
            break
        out.insert(0, leaf)
        if curr_path == head:
            break
        curr_path = head
    return out


def fmt_num(num: int) -> str:
    return "{:,}".format(num)


def main() -> None:
    tree: Dict[str, Any] = dict(name="root", size=0, children={})
    for root, _, files in os.walk(".", topdown=True):
        split_paths(root)
        for name in files:
            fullpath = os.path.join(root, name)
            try:
                size = os.path.getsize(fullpath)
                path_lst = split_paths(fullpath)
                add_path(tree, path_lst, size)
            except FileNotFoundError:
                continue
            except OSError:
                continue

    top: dict = tree["children"]["."]
    top_sizes = []
    total_size = 0
    for _, node in top["children"].items():
        n = node["size"]
        total_size += n
        top_sizes.append((n, node["name"]))

    top_sizes.sort()
    top_sizes.reverse()

    max_nm_len = 0
    for _, name in top_sizes:
        if len(name) > max_nm_len:
            max_nm_len = len(name)
    lines = [f"Total size: {fmt_num(total_size)}:"]
    for size, name in top_sizes:
        nm = name + ": ".ljust(max_nm_len + 2 - len(name), " ")
        perc_num = "{:.1%}".format(size / total_size)
        if os.path.isfile(name):
            nm = "F " + nm
        else:
            nm = "D " + nm
        num = fmt_num(size)
        lines.append(f"  {nm} {num} ({perc_num})")
    print("\n".join(lines))


if __name__ == "__main__":
    main()
    sys.exit(0)



# pylint: skip-file

import os
import sys
from pathlib import Path


def main():
    if len(sys.argv) != 2:
        print(f"Expected one arg, but got {len(sys.argv)} instead.")
        sys.exit(1)
    chop_seconds = int(input("how many seconds? "))
    filename = sys.argv[1:2][0]
    if not os.path.exists(filename):
        print(f"{filename} does not exist")
        sys.exit(1)
    out_path = Path(filename).with_stem(f"chopped_{filename}").with_suffix(".mp4")
    cmd = f'ffmpeg -i "{filename}" -ss {chop_seconds} -c copy "{out_path}"'
    os.system(cmd)


if __name__ == "__main__":
    main()



# pylint: skip-file

"""
    Updater for zcmds.
"""

import sys
import os

SELF_DIR = os.path.dirname(__file__)


def main() -> None:
    if sys.platform == "darwin":
        from zcmds.install.darwin.update import main as darwin_update

        darwin_update()
    elif sys.platform == "win32":
        from zcmds.install.win32.update import main as win32_update

        win32_update()
    elif sys.platform == "linux":
        from zcmds.install.linux.update import main as linux_update

        linux_update()
    else:
        raise ValueError("Unhandled platform " + sys.platform)


if __name__ == "__main__":
    main()



# pylint: skip-file

import os
import sys
import argparse


def sanitize(s: str) -> str:
    return s.replace(":", "-")


def stripext(s: str) -> str:
    return os.path.splitext(s)[0]


def main():

    parser = argparse.ArgumentParser(
        description="Cuts clips from local files.\n",
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument("input", help="input")
    parser.add_argument("--start_timestamp", help="start of the clip")
    parser.add_argument("--length", help="length of the clip")
    parser.add_argument("--outname", help="output name of the file")
    parser.add_argument(
        "--crf", default="18", type=int, help="crf quality of the file."
    )
    args = parser.parse_args()

    infile = args.input
    start_timestamp = args.start_timestamp or input("start_timestamp: ")
    length = args.length or input("length (seconds): ")
    output_path = args.outname or stripext(infile) + "_clip.mp4"
    crf = args.crf  # The amount of quality, 0 is lossless and 50 is shit.

    print(infile)
    if not os.path.exists(infile):
        print(f"{infile} does not exist")
        sys.exit(1)
    cmd = f'ffmpeg -i "{infile}" -c:v libx264 -crf {crf} -ss {start_timestamp} -t {length} "{output_path}"'
    os.system(cmd)
    if not os.path.exists(output_path):
        print(f"Error, did not generate {output_path}")
    else:
        print(f"Generated: {output_path}")


if __name__ == "__main__":
    main()



# pylint: skip-file

import argparse
import os
import shutil
import subprocess
import sys
import tempfile


def _make_output_name(in_file: str) -> str:
    pathname, extension = os.path.splitext(in_file)
    out_file = f"{pathname}_no_audio{extension}"
    return out_file


def main() -> None:
    parser = argparse.ArgumentParser(
        description="Removes an audio track from a source\n",
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument("input", help="Filename to strip audio out of")
    parser.add_argument(
        "output",
        nargs="?",
        help="Optional output file, other wise a new file is created with <filename>_strip_audio.<ext>",
    )
    args = parser.parse_args()
    input_file = args.input or input("")
    output = args.output or _make_output_name(input_file)

    error_code = 0
    with tempfile.TemporaryDirectory() as tmpdir:
        # temporary directory allows use the use case of input and output
        # name being the same.
        tempout = os.path.join(tmpdir, "file.mp4")
        cmd = f"static_ffmpeg -i {input_file} -c copy -an {tempout}"
        try:
            subprocess.check_output(cmd, shell=True)
            shutil.copyfile(tempout, output)
        except subprocess.CalledProcessError as cpe:
            print(
                f"{__file__}: Error while processing {cmd} because of {cpe}"
                f"\nSTDOUT\n#################\n{cpe.output}"
            )
            error_code = cpe.returncode
    print(f"\n\nCompleted! Output file: {output}")
    sys.exit(error_code)


if __name__ == "__main__":
    main()



# pylint: skip-file

"""
    Main entry point into the zcmds console program.
"""

import os
import sys

from zcmds.cmds.common.update import main as update_main

SELF_DIR = os.path.dirname(__file__)
BASE_DIR = os.path.abspath(os.path.join(SELF_DIR, "..", ".."))
BIN_DIR = os.path.join(BASE_DIR, "bin")


def main():
    first_run = not os.path.isdir(BIN_DIR)
    os.makedirs(BIN_DIR, exist_ok=True)
    if "update" in sys.argv or first_run:
        update_main()
    cmds = os.listdir(BIN_DIR)
    cmds = [
        os.path.splitext(c)[0] for c in cmds if c.endswith(".bat") or c.endswith(".exe")
    ]
    cmds.append("ytclip")
    cmds = list(set(cmds))
    if "update" in cmds:
        cmds.remove("update")
    cmds.append("zcmds update")
    cmds.append("zcmds_install")
    cmds.sort()
    print(f"BIN_DIR: {BIN_DIR}:\nCommands:\n  " + "\n  ".join(cmds))
    return


if __name__ == "__main__":
    main()



# pylint: disable=invalid-name

"""
    Update mechanism for darwin (MacOS)
"""

import os
import shutil
import sys

SELF_DIR = os.path.dirname(__file__)
BASE_DIR = os.path.abspath(os.path.join(SELF_DIR, "..", ".."))
COMMON_DIR = os.path.join(BASE_DIR, "cmds", "common")
BIN_DIR = os.path.abspath(os.path.join(BASE_DIR, "bin"))


def gen_macos_cmds():
    """Generates commands for MacOS."""
    common_cmds = [
        os.path.abspath(os.path.join(COMMON_DIR, f)) for f in os.listdir(COMMON_DIR)
    ]
    macos_cmds = [
        os.path.abspath(os.path.join(SELF_DIR, f)) for f in os.listdir(SELF_DIR)
    ]
    all_cmds = common_cmds + macos_cmds
    all_cmds = [cmd for cmd in all_cmds if cmd.endswith(".py")]
    shutil.rmtree(BIN_DIR, ignore_errors=True)
    os.makedirs(BIN_DIR, exist_ok=True)
    cmd_set = set([])
    for cmd in all_cmds:
        if cmd in cmd_set:
            sys.stderr.write(
                f"Warning, duplicate found for {os.path.basename(cmd)}, skipping."
            )
        else:
            cmd_set.add(cmd)
        print("making command for " + cmd)
        cmd_name = os.path.basename(cmd)
        out_cmd = os.path.join(BIN_DIR, cmd_name)[0:-3]  # strips .py extension
        with open(out_cmd, encoding="utf-8", mode="wt") as f:
            f.write("python3 " + cmd + "\n")
        # Allow execution on the commands.
        os.system("chmod +x " + out_cmd)


def add_cmds_to_path() -> None:
    """Adds MacOS path to the current environment."""
    needle = f"export PATH=$PATH:{BIN_DIR}"
    bash_profile_file = os.path.expanduser(os.path.join("~", ".bash_profile"))
    if not os.path.exists(bash_profile_file):
        bash_profile_file = os.path.expanduser(os.path.join("~", ".bashrc"))
    with open(bash_profile_file, encoding="utf-8", mode="rt") as fd:
        bash_profile = fd.read()
    if needle in bash_profile:
        return
    print(f"Attempting to install {BIN_DIR} in {bash_profile_file}")
    lines = bash_profile.splitlines()
    last_path_line = 0
    for i, line in enumerate(lines):
        if "export PATH=" in line:
            last_path_line = i
            continue
    lines.insert(last_path_line + 1, needle)
    # print('\n'.join(lines))
    out_file = "\n".join(lines) + "\n"
    with open(bash_profile_file, encoding="utf-8", mode="wt") as fd:
        fd.write(out_file)

    with open(bash_profile_file, encoding="utf-8", mode="rt") as fd:
        bash_profile = fd.read()
    if needle not in bash_profile:
        raise ValueError(f"{needle} could not be installed into {bash_profile_file}")


def add_python_key_bindings():
    """Adds keybindings to make python development work much better."""
    target_file = os.path.join(
        os.path.expanduser("~"), "Library", "Keybindings", "DefaultKeyBinding.dict"
    )
    if not os.path.exists(target_file):
        os.makedirs(os.path.dirname(target_file), exist_ok=True)
        with open(target_file, encoding="utf-8", mode="w") as filed:
            filed.write("")
    src_file = os.path.join(SELF_DIR, "macOS_key_bindings.dict")
    with open(src_file, encoding="utf-8", mode="rt") as fd:
        src_file_content = fd.read()
    if not os.path.exists(target_file):
        with open(target_file, encoding="utf-8", mode="rt") as fd:
            fd.write(src_file_content)
            return
    else:
        with open(target_file, encoding="utf-8", mode="rt") as fd:
            target_file_content = fd.read()
        if target_file_content != src_file_content:
            sys.stderr.write(f"Please manually merge {src_file} with {target_file}\n")


def main():
    """Main entry point."""
    gen_macos_cmds()
    add_cmds_to_path()
    add_python_key_bindings()


if __name__ == "__main__":
    main()



# pylint: skip-file

import os
import sys
import argparse


def stripext(s: str) -> str:
    return os.path.splitext(s)[0]


def main():
    parser = argparse.ArgumentParser(
        description="Extracts a video frame.\n",
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument("input", help="input")
    parser.add_argument("--timestamp", help="start of the clip")
    parser.add_argument("--length", help="length of the clip", default="1")
    parser.add_argument("--outname", help="output name of the file")
    args = parser.parse_args()
    infile = args.input
    timestamp = args.timestamp or input("timestamp: ")
    length = args.length
    if args.outname:
        output_path = os.path.splitext(args.outname)[0]
    else:
        output_path = os.path.splitext(args.input)[0]
    output_path = output_path + "_imgs"
    if not os.path.exists(infile):
        print(f"{infile} does not exist")
        sys.exit(1)

    if not os.path.isdir(output_path):
        os.makedirs(output_path)
    # ffmpeg -i foo.avi -r 1 -s WxH -f image2 foo-%03d.jpeg -ss -frames:v
    file_output_fmt = f'"{output_path}/%03d.jpg"'

    cmd = (
        f'ffmpeg -i "{infile}" -ss {timestamp} -t {length} -f image2 {file_output_fmt}'
    )
    print(f"Executing:\n  {cmd}\n")
    os.system(cmd)
    if not os.path.exists(output_path):
        print(f"Error, did not generate {output_path}")
        print(f"Error from cmd:\n  {cmd}\n")
    else:
        print(f"Generated images are in directory: {output_path}")
        if sys.platform == "win32":
            os.system(f"start explorer {output_path}")


if __name__ == "__main__":
    main()



# pylint: skip-file

import os
import sys
from pathlib import Path


def main():
    if len(sys.argv) != 2:
        print(f"Expected two args, but got {len(sys.argv)} instead.")
        sys.exit(1)
    filename = sys.argv[1:2][0]
    if not os.path.exists(filename):
        print(f"{filename} does not exist")
        sys.exit(1)
    out_left_path = Path("left_" + filename).with_suffix(".wav")
    out_right_path = Path("right_" + filename).with_suffix(".wav")
    cmd0 = f'ffmpeg -i %1 -filter_complex "[0:a]channelsplit=channel_layout=stereo:channels=FR[right]" -map "[right]" {out_right_path}'
    cmd1 = f'ffmpeg -i %1 -filter_complex "[0:a]channelsplit=channel_layout=stereo:channels=FL[left]" -map "[left]" {out_left_path}'
    os.system(cmd0)
    os.system(cmd1)


if __name__ == "__main__":
    main()



# pylint: skip-file

import os
import sys


def main():
    if len(sys.argv) != 2:
        print(f"Expected two args, but got {len(sys.argv)} instead.")
        sys.exit(1)
    filename = sys.argv[1:2][0]
    if not os.path.exists(filename):
        print(f"{filename} does not exist")
        sys.exit(1)
    path, _ = os.path.splitext(filename)
    out_path = f"{path}_small.mp4"
    cmd = f'ffmpeg -i "{filename}" -vf scale=640:-1 -c:v libx264 -crf 19 "{out_path}"'
    os.system(cmd)


if __name__ == "__main__":
    main()



# pylint: skip-file
"""
    Installation/Update script for win32.
"""

import os
import sys
import shutil

from typing import List

from zcmds.util import win32_path_manip

SELF_DIR = os.path.abspath(os.path.dirname(__file__))
BASE_DIR = os.path.abspath(os.path.join(SELF_DIR, "..", ".."))
COMMON_DIR = os.path.join(BASE_DIR, "cmds", "common")
BIN_DIR = os.path.abspath(os.path.join(BASE_DIR, "bin"))

# Sanity checks
assert os.path.exists(BASE_DIR), f"could not find {BASE_DIR}"
assert os.path.exists(COMMON_DIR), f"could not find {COMMON_DIR}"


def gen_win_cmds() -> None:
    """Generates windows commands."""
    common_cmds = os.listdir(COMMON_DIR)
    if "zcmds" in common_cmds:
        common_cmds.remove("zcmds")  # This one is built in.
    common_cmds = [os.path.abspath(os.path.join(COMMON_DIR, f)) for f in common_cmds]
    win32_cmds = [
        os.path.abspath(os.path.join(SELF_DIR, f)) for f in os.listdir(SELF_DIR)
    ]
    all_cmds = common_cmds + win32_cmds
    all_cmds = [
        cmd
        for cmd in all_cmds
        if cmd.endswith(".py") or cmd.endswith(".bat")
    ]
    shutil.rmtree(BIN_DIR, ignore_errors=True)
    os.makedirs(BIN_DIR, exist_ok=True)
    cmd_set = set([])
    for cmd in all_cmds:
        if cmd in cmd_set:
            sys.stderr.write(
                f"Warning, duplicate found for {os.path.basename(cmd)}, skipping."
            )
        else:
            cmd_set.add(cmd)
        if cmd.endswith(".py"):
            print("making command for " + cmd)
            cmd_name = os.path.basename(cmd)
            out_cmd = os.path.join(BIN_DIR, cmd_name)[0:-3] + ".bat"  # swap .py -> .bat
            with open(
                out_cmd, encoding="utf-8", mode="wt"
            ) as f:  # pylint: disable=invalid-name
                f.write(f"python {cmd} %1 %2 %3 %4 %5 %6 %7 %8 %9\n")

        elif cmd.endswith(".bat"):
            cmd_name = os.path.basename(cmd)
            out_cmd = os.path.join(BIN_DIR, cmd_name)
            print(f"copying command {cmd} -> {out_cmd}")
            shutil.copy(cmd, out_cmd)
        else:
            print(f"Unexpected command type: {cmd}")
    # git-bash hack, which won't execute run.bat without specifying the
    # entire name. Work around is to construct a text based exe file-
    # trampoline.
    for file_name in os.listdir(BIN_DIR):
        if file_name.endswith(".bat"):
            if file_name == "update.bat":
                # Hack-fix: exclude update.exe to prevent windows error.
                continue
            file = os.path.join(BIN_DIR, file_name)
            exe_file: str = os.path.splitext(file)[0] + ".exe"
            with open(exe_file, encoding="utf-8", mode="wt") as filed:
                filed.write(f'{file_name} "$@"')


def is_cmd_path_installed() -> bool:
    """Detects if the path is already installed."""
    all_paths: List[str] = win32_path_manip.read_user_path()
    return BIN_DIR in all_paths


def add_cmds_to_path() -> None:
    """Adds commands to the Win32 path, if it's not already installed."""
    if is_cmd_path_installed():
        print(f"Already found {BIN_DIR} in %PATH%")
        return
    print(f"\nDid not found {BIN_DIR} in %PATH%, adding...\n")
    win32_path_manip.append_user_path_if_not_exist(BIN_DIR)


def main():
    """Entry point."""
    gen_win_cmds()
    add_cmds_to_path()


if __name__ == "__main__":
    main()



"""
    Functions to allow manipulation of the windows %PATH% and have it take effect
    to the open command windows.
"""

import argparse
import ctypes
import winreg
from typing import List

REG_PATH_ENV = r"Environment"


def set_user_path(path_value: str) -> bool:
    """Sets the user path and broadcasts the update to all processes."""
    try:
        winreg.CreateKey(winreg.HKEY_CURRENT_USER, REG_PATH_ENV)
        registry_key = winreg.OpenKey(
            winreg.HKEY_CURRENT_USER, REG_PATH_ENV, 0, winreg.KEY_WRITE
        )
        winreg.SetValueEx(registry_key, "Path", 0, winreg.REG_SZ, path_value)
        winreg.CloseKey(registry_key)
        # Now broadcast change to all windows.
        WM_SETTINGCHANGE = 0x1A  # pylint: disable=invalid-name
        HWND_BROADCAST = 0xFFFF  # pylint: disable=invalid-name
        SMTO_ABORTIFHUNG = 0x0002  # pylint: disable=invalid-name
        result = ctypes.c_long()
        SendMessageTimeoutW = (  # pylint: disable=invalid-name
            ctypes.windll.user32.SendMessageTimeoutW
        )
        SendMessageTimeoutW(
            HWND_BROADCAST,
            WM_SETTINGCHANGE,
            0,
            "Environment",
            SMTO_ABORTIFHUNG,
            5000,
            ctypes.byref(result),
        )
        return True
    except WindowsError as err:
        print(f"{err}")
        return False


def read_user_path() -> List[str]:
    """Reads the user path from the registry."""
    try:
        registry_key = winreg.OpenKey(
            winreg.HKEY_CURRENT_USER, REG_PATH_ENV, 0, winreg.KEY_READ
        )
        path, _ = winreg.QueryValueEx(registry_key, "Path")
        winreg.CloseKey(registry_key)
        path_list = [e for e in path.split(";") if e]
        return path_list
    except WindowsError as err:
        print(f"{err}")
        return []


def append_user_path_if_not_exist(path: str):
    """Conditionally appends the path, if it doesn't already exist.."""
    all_paths: List[str] = read_user_path()
    if path in all_paths:
        return False
    all_paths_str = ";".join(all_paths)
    all_paths_str = f"{all_paths_str};{path}"
    set_user_path(all_paths_str)
    return True


def main():
    """Main entry point for the command line version of this tool."""
    parser = argparse.ArgumentParser(description="Win32 path management")
    parser.add_argument("--add_path", required=True)
    args = parser.parse_args()
    added = append_user_path_if_not_exist(args.add_path)
    if added:
        print("Added path")
    else:
        print("Already added.")


if __name__ == "__main__":
    main()



import os
import sys
from shutil import rmtree

from setuptools import Command, find_packages, setup

# The directory containing this file
HERE = os.path.dirname(__file__)

NAME = "zcmds"
DESCRIPTION = "Command to download a video and cut out a clip."
URL = f"https://github.com/zackees/{NAME}"
EMAIL = "dont@email.me"
AUTHOR = "Zach Vorhies"
REQUIRES_PYTHON = ">=3.6.0"
VERSION_FILE = os.path.join(HERE, NAME, "version.py")
README_FILE = os.path.join(HERE, "README.md")
REQUIREMENTS_FILE = os.path.join(HERE, "requirements.txt")

# The text of the README file
with open(README_FILE, encoding="utf-8", mode="rt") as fd:
    LONG_DESCRIPTION = fd.read()

with open(REQUIREMENTS_FILE, encoding="utf-8", mode="rt") as fd:
    REQUIREMENTS = [line.strip() for line in fd.readlines() if line.strip()]

with open(VERSION_FILE, encoding="utf-8", mode="rt") as fd:
    for line in fd.readlines():
        if "VERSION" in line:
            if "#" in line:  # Remove comments
                line = line.split("#")[0]
            VERSION = line.split("=")[1].strip(" \n\"")
            print(VERSION)
            break

class UploadCommand(Command):
    """Support setup.py upload."""

    description = "Build and publish the package."
    user_options = []

    @staticmethod
    def status(s):
        pass

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status("Removing previous builds…")
            rmtree(os.path.join(HERE, "dist"))
        except OSError:
            pass

        self.status("Building Source and Wheel (universal) distribution…")
        os.system('"{0}" setup.py sdist bdist_wheel --universal'.format(sys.executable))

        self.status("Uploading the package to PyPI via Twine…")
        os.system("twine upload dist/*")

        self.status("Pushing git tags…")
        os.system("git tag v{0}".format(VERSION))
        os.system("git push --tags")

        sys.exit()


setup(
    name=NAME,
    python_requires=REQUIRES_PYTHON,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    url=URL,
    author="Zach Vorhies",
    author_email="dont@email.me",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.9",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX",
        "Operating System :: MacOS :: MacOS X",
        "Environment :: Console",
    ],
    install_requires=REQUIREMENTS,

    entry_points={
        "console_scripts": [
            "zcmds_install = zcmds.cmds.common.update:main",
            "zcmds = zcmds.cmds.common.zcmds:main",
        ],
    },
    packages=find_packages(exclude=["tests", "*.tests", "*.tests.*", "tests.*"]),
    package_data={},
    include_package_data=True,
    extras_require={
        "test": ["pytest"],
    },
    cmdclass={
        "upload": UploadCommand,
    },
)



"""
    Helper functions for file based operations.
"""

import argparse
import fnmatch
import os


def get_search_args(require_replace_args=False):
    """Generates an ArgumentParser and returns the args."""
    parser = argparse.ArgumentParser()
    parser.add_argument("--cur_dir", default=os.curdir)
    parser.add_argument("--file_pattern", default=None)
    parser.add_argument("--search_string", default=None)
    parser.add_argument("--replace_string", default=None)
    args = parser.parse_args()
    if args.file_pattern is None:
        args.file_pattern = input("file search pattern: ")
    if args.search_string is None:
        args.search_string = input("search string: ")
    if require_replace_args and args.replace_string is None:
        args.replace_string = input("replace string: ")
    return args


def iter_matching_files(cur_dir, file_pattern, text_search_string=None):
    """Generates an iterator for matching files."""
    for path, dirs, files in os.walk(cur_dir):  # pylint: disable=unused-variable
        for f in files:  # pylint: disable=invalid-name
            if fnmatch.fnmatch(f, file_pattern):
                full_path = os.path.join(path, f)
                if text_search_string is None:
                    yield full_path
                else:
                    with open(
                        full_path, encoding="utf-8"
                    ) as fd:  # pylint: disable=invalid-name
                        file_data = fd.read()
                    if text_search_string in file_data:
                        yield full_path


def replace_in_file(file_path, search_text, replace_text):
    """Replaces all occurences with replace_text."""
    with open(file_path, encoding="utf-8") as fd:  # pylint: disable=invalid-name
        file_data = fd.read()
    file_data = file_data.replace(search_text, replace_text)
    with open(
        file_path, encoding="utf-8", mode="w"
    ) as fd:  # pylint: disable=invalid-name
        fd.write(file_data)



# pylint: skip-file

from pdf2image import convert_from_path  # type: ignore
import os


def main() -> None:
    infile = input("input pdf: ")
    page_start = int(input("first_page: "))
    page_last = int(input("last page: "))
    dpi = int(input("dpi: "))

    save_dir = os.path.splitext(infile)[0]
    os.makedirs(save_dir, exist_ok=True)
    infile = save_dir + ".pdf"
    images = convert_from_path(
        infile, first_page=page_start, last_page=page_last, dpi=dpi
    )
    for i, image in enumerate(images):
        output_png = os.path.join(save_dir, f"{i+page_start}.png")
        if os.path.isfile(output_png):
            os.remove(output_png)
        print(f"Writing: {output_png}")
        image.save(output_png)
        if not os.path.isfile(output_png):
            print(f"Error: could not save {output_png}")


if __name__ == "__main__":
    main()



# pylint: skip-file

import shlex
import subprocess


DNS_BACKBONE = "8.8.8.8"


def ParsePingOutput(line):
    time_ms = "".join(shlex.split(line)[6:7])[5:]
    if time_ms:
        return str(int(float(time_ms) + 0.5))
    else:
        return line


def RunMultiPing(ip_array):
    procs = []
    for ip in ip_array:
        cmd_str = "ping -n -c 1 -t 1 " + ip
        p = subprocess.Popen(cmd_str, shell=True, stdout=subprocess.PIPE)
        procs.append(p)
    out = []
    for p in procs:
        p.stdout.readline()  # Throw away first readline
        line = p.stdout.readline().rstrip()
        if line:
            out.append(ParsePingOutput(line))
        else:
            out.append("NET_ERR")
        p.wait()
    return out


def GetTraceRoute(dest_ip):
    """Returns a list of routes to 8.8.8.8 in [(name, ip), ...] form."""
    trace_route = []
    cmd = "traceroute " + dest_ip
    try:
        p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
        for line in iter(p.stdout.readline, ""):
            if line.startswith("  "):
                continue
            line = line.rstrip().replace("* ", "")
            tokens = shlex.split(line)
            try:
                name = tokens[1]
                ip = tokens[2].replace("(", "").replace(")", "")
                print(f"found: {name} ({ip})")
                trace_route.append((name, ip))
            except IndexError:
                print(f"Error tokenizing {line}, skipping")
    finally:
        p.kill()
    return trace_route


def Main():
    def IsIpPingable(ip):
        p = subprocess.Popen("ping -c 5 -t 1 " + ip, shell=True, stdout=subprocess.PIPE)
        for line in iter(p.stdout.readline, ""):
            if "time=" in line:
                p.wait()
                return True
        print(ip + " is invalid")
        return False

    trace_route = GetTraceRoute("8.8.8.8")
    # Filter out ip's that won't ping fast.
    trace_route = [e for e in trace_route if IsIpPingable(e[1])]

    def MakeDataRow():
        ips = [e[1] for e in trace_route]
        times = RunMultiPing(ips)
        out_str = ""
        for t in times:
            out_str += "{0: ^16s} | ".format(t)
        return out_str

    def MakeHeaderRow():
        def sizeStr(s, n):
            if len(s) < n:
                return s
            else:
                s = s[0 : n - 3] + "..."
                return s

        tmp = []
        out_str = ""
        for name, ip in trace_route:
            tmp.append(ip)
            out_str += "{0: ^16s} | ".format(sizeStr(name, 16))
        out_str += "\n"
        for name, ip in trace_route:
            tmp.append(ip)
            out_str += "{0: ^16s} | ".format(ip)
        return "\n" + out_str + "\n"

    while True:
        print(MakeHeaderRow())
        for i in range(0, 50):
            row = MakeDataRow()
            print(row)


if __name__ == "__main__":
    try:
        Main()
    except KeyboardInterrupt:
        pass
